package main

import (
	log "github.com/sirupsen/logrus"
	"net/http"
	"os"
	"os/signal"
	"random.chars.jp/git/image-board/store"
	"syscall"
)

var (
	instance   *store.Store
	server     = http.Server{}
	executable string
)

var (
	r bool
	d bool
)

func init() {
	setFormatter()
	var err error
	executable, err = os.Executable()
	if err != nil {
		log.Warnf("Error while obtaining executable path, %s. Restarting will no longer work.", err)
	}
}

func main() {

	// Setup configuration stuff
	configSetup()

	// Initial config update
	setLevel()

	// Open store
	openStore()

	// Set up web
	webSetup()

	// Configure listener
	listenerSetup()

	// Register API and web handlers
	registerAPI()
	registerWebpage()

	// Signal handling
	signalChannel := make(chan os.Signal, 1)
	signal.Notify(signalChannel, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, os.Interrupt, os.Kill)
	go func() {
		// Cleanup on function return
		defer func() { cleanup(false) }()
		for {
			currentSignal := <-signalChannel
			switch currentSignal {
			case os.Interrupt:
				println()
				log.Info("Gracefully exiting.")
				return
			default:
				log.Info("Gracefully exiting.")
				return
			}
		}
	}()

	// Start server
	runWebServer()

	// Restart if needed
	if r {
		restart()
	}
}

func openStore() {
	path := systemConfig["store"].(string)
	single := parseBool(systemConfig["single-user"])
	private := parseBool(systemConfig["private"])

	instance = store.New(path, single, private)
	if instance == nil {
		log.Fatalf("Error initializing store.")
	}
	log.Infof("Store opened on %s revision %v compat %v.", path, instance.Revision, instance.Compat)
	info := instance.User(instance.InitialUser)
	if info.Snowflake == instance.InitialUser {
		log.Infof("Initial user ID %s secret %s.", info.Snowflake, info.Secret)
		if instance.UserPasswordValidate(info.Snowflake, "initial") {
			log.Warnf("Initial user still has the initial password.")
		}
	} else {
		if single {
			log.Fatal("Instance has no initial user, single user mode unavailable.")
		}
	}
	if single {
		log.Info("Server running in single user mode, all operations are performed as the initial user.")
	} else if private {
		log.Info("Server running in private mode, all operations will require authentication.")
	}
}
